@@ -5,6 +5,8 @@
 #' @importFrom stats qnorm
 #' @importFrom stats as.formula
 #' @importFrom stats terms
+#' @importFrom stats reformulate
+#' @importFrom survey svytotal
 #' @importFrom MASS ginv
 #' @import Rcpp
 #' @importFrom Rcpp evalCpp
@@ -66,6 +68,8 @@ nonprobIPW <- function(selection,
       data = data,
       svydesign = svydesign
     )
+    prob_totals <- svytotal(selection, svydesign)
+    prob_pop_totals <- c(sum(weights(svydesign)), prob_totals)
     # y_nons <- model$y_nons
 
     X_nons <- model$X_nons
@@ -165,7 +169,7 @@ nonprobIPW <- function(selection,
       method_selection = method_selection
     )
     theta_hat <- selection_model$theta_hat
-    # grad <- est_method_obj$grad
+    grad <- selection_model$grad
     hess <- selection_model$hess
     var_cov1 <- selection_model$var_cov1
     var_cov2 <- selection_model$var_cov2
@@ -275,6 +279,7 @@ nonprobIPW <- function(selection,
       pop_totals <- model$pop_totals[idx]
     }
 
+    prob_pop_totals <- pop_totals
     if (is.null(start_selection)) {
       if (control_selection$start_type == "glm") {
         start_selection <- start_fit(
@@ -294,6 +299,9 @@ nonprobIPW <- function(selection,
           method_selection = method_selection,
           start = 0,
           maxit = maxit,
+          nleqslv_method = control_selection$nleqslv_method,
+          nleqslv_global = control_selection$nleqslv_global,
+          nleqslv_xscalm = control_selection$nleqslv_xscalm,
           pop_totals = pop_totals[1]
         )$theta_h)
         start_selection <- c(start_h, rep(0, ncol(X) - 1))
@@ -309,6 +317,9 @@ nonprobIPW <- function(selection,
       method_selection = method_selection,
       start = start_selection,
       maxit = maxit,
+      nleqslv_method = control_selection$nleqslv_method,
+      nleqslv_global = control_selection$nleqslv_global,
+      nleqslv_xscalm = control_selection$nleqslv_xscalm,
       pop_totals = pop_totals
     ) # theta_h estimation for h_x == 2 is equal to the main method for theta estimation
 
@@ -529,6 +540,8 @@ nonprobIPW <- function(selection,
   if (is.null(pop_size)) pop_size <- N # estimated pop_size
   names(pop_size) <- "pop_size"
   names(ys) <- all.vars(outcome_init[[2]])
+  est_totals <- colSums(X_nons * as.vector(weights_nons))
+  names(prob_pop_totals) <- colnames(X_nons)
 
   boot_sample <- if (control_inference$var_method == "bootstrap" & control_inference$keep_boot) {
     boot_obj$stat
@@ -549,9 +562,20 @@ nonprobIPW <- function(selection,
     aic = selection_model$aic,
     weights = as.vector(weights_nons),
     prior.weights = weights,
+    est_totals = est_totals,
+    pop_totals = pop_totals,
     formula = selection,
     df_residual = selection_model$df_residual,
     log_likelihood = selection_model$log_likelihood,
+    method_selection = method_selection,
+    hessian = hess,
+    gradient = grad,
+    method = est_method,
+    prob_der = ps_nons_der,
+    prob_rand = ps_rand,
+    prob_rand_est = est_ps_rand,
+    prob_rand_est_der = est_ps_rand_der,
+    h_function = h,
     cve = if (control_inference$vars_selection == TRUE) {
       cve_selection
     } else {
@@ -561,8 +585,10 @@ nonprobIPW <- function(selection,
 
   structure(
     list(
+      data = data,
       X = if (isTRUE(x)) X else NULL,
       y = if (isTRUE(y)) ys else NULL,
+      R = R,
       prob = prop_scores,
       weights = as.vector(weights_nons),
       control = list(
@@ -575,8 +601,11 @@ nonprobIPW <- function(selection,
       nonprob_size = n_nons,
       prob_size = n_rand,
       pop_size = pop_size,
+      pop_totals = prob_pop_totals,
+      outcome = NULL,
       selection = SelectionList,
-      boot_sample = boot_sample
+      boot_sample = boot_sample,
+      svydesign = if (is.null(pop_totals)) svydesign else NULL # TODO to customize if pop_totals only
     ),
     class = c("nonprobsvy", "nonprobsvy_ipw")
   )
